@@ -32,8 +32,8 @@ public class DistanceNoAnnotations {
     }
 
     public DistanceNoAnnotations(String amount) {
-        amount = amount.substring(0, amount.length() - 1);
-        this.amount = Integer.parseInt(amount);
+        String amt = amount.substring(0, amount.length() - 1);
+        this.amount = Integer.parseInt(amt);
     }
 
     public String print() {
