@@ -1,5 +1,5 @@
 /*
- *  Copyright 2010 Stephen Colebourne
+ *  Copyright 2010-present Stephen Colebourne
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -27,8 +27,8 @@ public class DistanceToStringException {
 
     @FromString
     public static DistanceToStringException parse(String amount) {
-        amount = amount.substring(0, amount.length() - 1);
-        return new DistanceToStringException(Integer.parseInt(amount));
+        String amt = amount.substring(0, amount.length() - 1);
+        return new DistanceToStringException(Integer.parseInt(amt));
     }
 
     public DistanceToStringException(int amount) {
