@@ -1,5 +1,5 @@
 /*
- *  Copyright 2010 Stephen Colebourne
+ *  Copyright 2010-present Stephen Colebourne
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -22,8 +22,8 @@ public class SubMethodMethod extends DistanceMethodMethod {
 
     @FromString
     public static SubMethodMethod parse(String amount) {
-        amount = amount.substring(0, amount.length() - 1);
-        return new SubMethodMethod(Integer.parseInt(amount));
+        String amt = amount.substring(0, amount.length() - 1);
+        return new SubMethodMethod(Integer.parseInt(amt));
     }
 
     public SubMethodMethod(int amount) {
