@@ -1,5 +1,5 @@
 /*
- *  Copyright 2010 Stephen Colebourne
+ *  Copyright 2010-present Stephen Colebourne
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -25,8 +25,8 @@ public class SuperFactorySuper {
 
     @FromString
     public static SuperFactorySuper parse(String amount) {
-        amount = amount.substring(0, amount.length() - 1);
-        int i = Integer.parseInt(amount);
+        String amt = amount.substring(0, amount.length() - 1);
+        int i = Integer.parseInt(amt);
         return i > 10 ? new SuperFactorySuper(i) : new SuperFactorySub(i);
     }
 
