@@ -17,6 +17,7 @@ package org.joda.convert;
 
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertFalse;
+import static org.junit.Assert.assertNull;
 import static org.junit.Assert.assertSame;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
@@ -148,11 +149,21 @@ public class TestStringConvert {
       assertEquals(Boolean.TRUE, StringConvert.INSTANCE.convertFromString(Boolean.TYPE, "true"));
     }
 
+    @Test
+    public void test_convertFromString_enumSubclass() {
+      assertEquals(ValidityCheck.VALID, StringConvert.INSTANCE.convertFromString(ValidityCheck.class, "VALID"));
+    }
+
     @Test
     public void test_convertFromString_inherit() {
         assertEquals(RoundingMode.CEILING, StringConvert.INSTANCE.convertFromString(RoundingMode.class, "CEILING"));
     }
 
+    @Test(expected=RuntimeException.class)
+    public void test_convertFromString_inheritNotSearchedFor() {
+        StringConvert.INSTANCE.convertFromString(AltCharSequence.class, "A");
+    }
+
     @Test
     public void test_convertFromString_null() {
         assertEquals(null, StringConvert.INSTANCE.convertFromString(Integer.class, null));
@@ -160,7 +171,7 @@ public class TestStringConvert {
 
     @Test(expected = IllegalArgumentException.class)
     public void test_convertFromString_nullClass() {
-        assertEquals(null, StringConvert.INSTANCE.convertFromString(null, "6"));
+        assertNull(StringConvert.INSTANCE.convertFromString(null, "6"));
     }
 
     //-----------------------------------------------------------------------
@@ -209,7 +220,7 @@ public class TestStringConvert {
         assertEquals("25m", test.convertToString(d));
         assertEquals(d.amount, test.convertFromString(DistanceMethodMethod.class, "25m").amount);
         TypedStringConverter<DistanceMethodMethod> conv = test.findTypedConverter(DistanceMethodMethod.class);
-        assertEquals(true, conv instanceof MethodsStringConverter<?>);
+        assertFromStringConverter(conv, MethodFromStringConverter.class);
         assertSame(conv, test.findConverter(DistanceMethodMethod.class));
         assertEquals(DistanceMethodMethod.class, conv.getEffectiveType());
         assertEquals(true, conv.toString().startsWith("RefectionStringConverter"));
@@ -222,7 +233,7 @@ public class TestStringConvert {
         assertEquals("25m", test.convertToString(d));
         assertEquals(d.amount, test.convertFromString(DistanceMethodMethodCharSequence.class, "25m").amount);
         TypedStringConverter<DistanceMethodMethodCharSequence> conv = test.findTypedConverter(DistanceMethodMethodCharSequence.class);
-        assertEquals(true, conv instanceof MethodsStringConverter<?>);
+        assertFromStringConverter(conv, MethodFromStringConverter.class);
         assertSame(conv, test.findConverter(DistanceMethodMethodCharSequence.class));
         assertEquals(DistanceMethodMethodCharSequence.class, conv.getEffectiveType());
         assertEquals(true, conv.toString().startsWith("RefectionStringConverter"));
@@ -235,7 +246,7 @@ public class TestStringConvert {
         assertEquals("25m", test.convertToString(d));
         assertEquals(d.amount, test.convertFromString(DistanceMethodConstructor.class, "25m").amount);
         TypedStringConverter<DistanceMethodConstructor> conv = test.findTypedConverter(DistanceMethodConstructor.class);
-        assertEquals(true, conv instanceof MethodConstructorStringConverter<?>);
+        assertFromStringConverter(conv, ConstructorFromStringConverter.class);
         assertSame(conv, test.findConverter(DistanceMethodConstructor.class));
         assertEquals(DistanceMethodConstructor.class, conv.getEffectiveType());
         assertEquals(true, conv.toString().startsWith("RefectionStringConverter"));
@@ -248,12 +259,25 @@ public class TestStringConvert {
         assertEquals("25m", test.convertToString(d));
         assertEquals(d.amount, test.convertFromString(DistanceMethodConstructorCharSequence.class, "25m").amount);
         TypedStringConverter<DistanceMethodConstructorCharSequence> conv = test.findTypedConverter(DistanceMethodConstructorCharSequence.class);
-        assertEquals(true, conv instanceof MethodConstructorStringConverter<?>);
+        assertFromStringConverter(conv, ConstructorFromStringConverter.class);
         assertSame(conv, test.findConverter(DistanceMethodConstructorCharSequence.class));
         assertEquals(DistanceMethodConstructorCharSequence.class, conv.getEffectiveType());
         assertEquals(true, conv.toString().startsWith("RefectionStringConverter"));
     }
 
+    @Test
+    public void test_convert_annotationMethodBridgeMethod() {
+        StringConvert test = new StringConvert();
+        HasCodeImpl d = new HasCodeImpl("CODE");
+        assertEquals("CODE", test.convertToString(d));
+        assertEquals(d.code, test.convertFromString(HasCodeImpl.class, "CODE").code);
+        TypedStringConverter<HasCodeImpl> conv = test.findTypedConverter(HasCodeImpl.class);
+        assertFromStringConverter(conv, ConstructorFromStringConverter.class);
+        assertSame(conv, test.findConverter(HasCodeImpl.class));
+        assertEquals(HasCodeImpl.class, conv.getEffectiveType());
+        assertEquals(true, conv.toString().startsWith("RefectionStringConverter"));
+    }
+
     @Test
     public void test_convert_annotationSubMethodMethod() {
         StringConvert test = new StringConvert();
@@ -261,7 +285,7 @@ public class TestStringConvert {
         assertEquals("25m", test.convertToString(d));
         assertEquals(d.amount, test.convertFromString(SubMethodMethod.class, "25m").amount);
         TypedStringConverter<SubMethodMethod> conv = test.findTypedConverter(SubMethodMethod.class);
-        assertEquals(true, conv instanceof MethodsStringConverter<?>);
+        assertFromStringConverter(conv, MethodFromStringConverter.class);
         assertEquals(SubMethodMethod.class, conv.getEffectiveType());
         assertSame(conv, test.findConverter(SubMethodMethod.class));
     }
@@ -273,7 +297,7 @@ public class TestStringConvert {
         assertEquals("25m", test.convertToString(d));
         assertEquals(d.amount, test.convertFromString(SubMethodConstructor.class, "25m").amount);
         TypedStringConverter<SubMethodConstructor> conv = test.findTypedConverter(SubMethodConstructor.class);
-        assertEquals(true, conv instanceof MethodConstructorStringConverter<?>);
+        assertFromStringConverter(conv, ConstructorFromStringConverter.class);
         assertEquals(SubMethodConstructor.class, conv.getEffectiveType());
         assertSame(conv, test.findConverter(SubMethodConstructor.class));
     }
@@ -285,7 +309,7 @@ public class TestStringConvert {
         assertEquals("25m", test.convertToString(d));
         assertEquals(d.amount, test.convertFromString(SuperFactorySuper.class, "25m").amount);
         TypedStringConverter<SuperFactorySuper> conv = test.findTypedConverter(SuperFactorySuper.class);
-        assertEquals(true, conv instanceof MethodsStringConverter<?>);
+        assertFromStringConverter(conv, MethodFromStringConverter.class);
         assertEquals(SuperFactorySuper.class, conv.getEffectiveType());
         assertSame(conv, test.findConverter(SuperFactorySuper.class));
     }
@@ -299,7 +323,7 @@ public class TestStringConvert {
         assertEquals(d.amount, fromStr.amount);
         assertEquals(true, fromStr instanceof SuperFactorySub);
         TypedStringConverter<SuperFactorySub> conv = test.findTypedConverter(SuperFactorySub.class);
-        assertEquals(true, conv instanceof MethodsStringConverter<?>);
+        assertFromStringConverter(conv, MethodFromStringConverter.class);
         assertEquals(SuperFactorySuper.class, conv.getEffectiveType());
         assertSame(conv, test.findConverter(SuperFactorySub.class));
     }
@@ -351,7 +375,7 @@ public class TestStringConvert {
         assertEquals("25m", test.convertToString(d));
         assertEquals(d.amount, test.convertFromString(DistanceWithFactory.class, "25m").amount);
         TypedStringConverter<DistanceWithFactory> conv = test.findTypedConverter(DistanceWithFactory.class);
-        assertEquals(true, conv instanceof MethodsStringConverter<?>);
+        assertFromStringConverter(conv, MethodFromStringConverter.class);
         assertEquals(DistanceWithFactory.class, conv.getEffectiveType());
         assertSame(conv, test.findConverter(DistanceWithFactory.class));
         assertEquals(true, conv.toString().startsWith("RefectionStringConverter"));
@@ -364,7 +388,7 @@ public class TestStringConvert {
         assertEquals("25g", test.convertToString(d));
         assertEquals(d.amount, test.convertFromString(Test1Class.class, "25g").amount);
         TypedStringConverter<Test1Class> conv = test.findTypedConverter(Test1Class.class);
-        assertEquals(true, conv instanceof MethodsStringConverter<?>);
+        assertFromStringConverter(conv, MethodFromStringConverter.class);
         assertEquals(Test1Class.class, conv.getEffectiveType());
         assertSame(conv, test.findConverter(Test1Class.class));
         assertEquals(true, conv.toString().startsWith("RefectionStringConverter"));
@@ -377,7 +401,7 @@ public class TestStringConvert {
         assertEquals("25g", test.convertToString(d));
         assertEquals(d.amount, test.convertFromString(Test2Class.class, "25g").amount);
         TypedStringConverter<Test2Class> conv = test.findTypedConverter(Test2Class.class);
-        assertEquals(true, conv instanceof MethodsStringConverter<?>);
+        assertFromStringConverter(conv, MethodFromStringConverter.class);
         assertEquals(Test2Interface.class, conv.getEffectiveType());
         assertSame(conv, test.findConverter(Test2Class.class));
         assertEquals(true, conv.toString().startsWith("RefectionStringConverter"));
@@ -390,7 +414,7 @@ public class TestStringConvert {
         assertEquals("25g", test.convertToString(d));
         assertEquals("25g", test.convertFromString(Test2Interface.class, "25g").print());
         TypedStringConverter<Test2Interface> conv = test.findTypedConverter(Test2Interface.class);
-        assertEquals(true, conv instanceof MethodsStringConverter<?>);
+        assertFromStringConverter(conv, MethodFromStringConverter.class);
         assertEquals(Test2Interface.class, conv.getEffectiveType());
         assertSame(conv, test.findConverter(Test2Interface.class));
         assertEquals(true, conv.toString().startsWith("RefectionStringConverter"));
@@ -403,7 +427,7 @@ public class TestStringConvert {
         assertEquals("25g", test.convertToString(d));
         assertEquals(d.amount, test.convertFromString(Test3Class.class, "25g").amount);
         TypedStringConverter<Test3Class> conv = test.findTypedConverter(Test3Class.class);
-        assertEquals(true, conv instanceof MethodsStringConverter<?>);
+        assertFromStringConverter(conv, MethodFromStringConverter.class);
         assertSame(conv, test.findConverter(Test3Class.class));
         assertEquals(Test3SuperClass.class, conv.getEffectiveType());
         assertEquals(true, conv.toString().startsWith("RefectionStringConverter"));
@@ -500,6 +524,14 @@ public class TestStringConvert {
         test.findConverter(DistanceTwoFromStringMethodAnnotations.class);
     }
 
+    //-----------------------------------------------------------------------
+    @Test
+    public void test_convertFromString_annotatedFromStringNoToString() {
+        StringConvert test = new StringConvert();
+        DistanceFromStringNoToString result = test.convertFromString(DistanceFromStringNoToString.class, "2m");
+        assertEquals(new DistanceFromStringNoToString(2), result);
+    }
+
     //-----------------------------------------------------------------------
     @Test
     public void test_convert_Enum_overrideDefaultWithConverter() {
@@ -598,7 +630,7 @@ public class TestStringConvert {
         assertEquals("Distance[25m]", test.convertToString(d));
         assertEquals(d.amount, test.convertFromString(DistanceNoAnnotations.class, "25m").amount);
         StringConverter<DistanceNoAnnotations> conv = test.findConverter(DistanceNoAnnotations.class);
-        assertEquals(true, conv instanceof MethodsStringConverter<?>);
+        assertFromStringConverter(conv, MethodFromStringConverter.class);
         assertSame(conv, test.findConverter(DistanceNoAnnotations.class));
     }
 
@@ -610,7 +642,7 @@ public class TestStringConvert {
         assertEquals("Distance[25m]", test.convertToString(d));
         assertEquals(d.amount, test.convertFromString(DistanceNoAnnotationsCharSequence.class, "25m").amount);
         StringConverter<DistanceNoAnnotationsCharSequence> conv = test.findConverter(DistanceNoAnnotationsCharSequence.class);
-        assertEquals(true, conv instanceof MethodsStringConverter<?>);
+        assertFromStringConverter(conv, MethodFromStringConverter.class);
         assertSame(conv, test.findConverter(DistanceNoAnnotationsCharSequence.class));
     }
 
@@ -671,7 +703,7 @@ public class TestStringConvert {
         assertEquals("Distance[25m]", test.convertToString(d));
         assertEquals(d.amount, test.convertFromString(DistanceNoAnnotationsCharSequence.class, "25m").amount);
         StringConverter<DistanceNoAnnotationsCharSequence> conv = test.findConverter(DistanceNoAnnotationsCharSequence.class);
-        assertEquals(true, conv instanceof MethodConstructorStringConverter<?>);
+        assertFromStringConverter(conv, ConstructorFromStringConverter.class);
         assertSame(conv, test.findConverter(DistanceNoAnnotationsCharSequence.class));
     }
 
@@ -683,7 +715,7 @@ public class TestStringConvert {
         assertEquals("Distance[25m]", test.convertToString(d));
         assertEquals(d.amount, test.convertFromString(DistanceNoAnnotations.class, "25m").amount);
         StringConverter<DistanceNoAnnotations> conv = test.findConverter(DistanceNoAnnotations.class);
-        assertEquals(true, conv instanceof MethodConstructorStringConverter<?>);
+        assertFromStringConverter(conv, ConstructorFromStringConverter.class);
         assertSame(conv, test.findConverter(DistanceNoAnnotations.class));
     }
 
@@ -723,4 +755,10 @@ public class TestStringConvert {
         assertEquals("StringConvert", new StringConvert().toString());
     }
 
+    private void assertFromStringConverter(StringConverter<?> conv, Class<?> expectedType) {
+        assertEquals(true, conv instanceof ReflectionStringConverter<?>);
+        Object obj = ((ReflectionStringConverter<?>) conv).fromString;
+        assertEquals(expectedType, obj.getClass());
+    }
+
 }
